# -*- coding: utf-8 -*-

import logging

from django.conf import settings
from django.contrib.auth.models import User, check_password
from django.db.models.signals import post_save
from django.contrib.auth import backends

from allauth.account.auth_backends import AuthenticationBackend
from allauth.account.utils import filter_users_by_email

from acomea.contact.models import Group
from acomea.ldap import ldap_models as mod
from acomea.ldap.signals import sync_ldap
from acomea.profilo.utils import create_acomeaprofilo_and_rac, get_or_create_contact

log = logging.getLogger("acomea.utenti")


class LDAPBackend(object):
    """
    Authenticate against ldap users
    """
    def authenticate(self, username=None, password=None):

        if not settings.LDAP_SERVER:
            return None

        conn = mod.get_conn()
        username = username.lower()
        if '@' in username:
            ldap_user = mod.get_user_by_email(username, conn=conn)
        else:
            ldap_user = mod.get_user_by_uid(username, conn=conn)

        password_is_valid = False
        created_from_ldap = False
        if ldap_user:

            try:
                dj_user = User.objects.get(username=ldap_user.uid.lower())
                created = False
            except User.DoesNotExist:
                if ldap_user.validate_password(password):
                    dj_user = User(
                        username=ldap_user.uid.lower(),
                        email=ldap_user.mail and ldap_user.mail.lower(),
                        first_name=ldap_user.first_name,
                        last_name=ldap_user.last_name,
                        is_superuser=False,
                    )
                    post_save.disconnect(sync_ldap, User)
                    dj_user.set_password(password)
                    dj_user.save() 
                    created = True
                    create_acomeaprofilo_and_rac(dj_user)
                    post_save.connect(sync_ldap, User)
                    created_from_ldap = True
                else:
                    return

            # If the user exists in ldap the password may be 'fake.*', in that
            # case we validate only if django stored password is valid
            if ldap_user.validate_password(password):
                password_is_valid = True
                post_save.disconnect(sync_ldap, User)
                dj_user.set_password(password)
                post_save.connect(sync_ldap, User)
            elif ldap_user.validate_password('fake!@#$'):
                log.info("User %s: found fake password in ldap, trying django stored one" % dj_user)
                if check_password(password, dj_user.password):
                    password_is_valid = True
                    ldap_user.set_password(password)
                    log.info("User %s: substituted fake password for user" % dj_user)

            mod.AcaUser.refresh_from_ldap(ldap_user=ldap_user, db_user=dj_user)
            get_or_create_contact(dj_user)

            if created_from_ldap:
                try:
                    group = Group.objects.get(id=settings.ACOMEA_LDAP_GROUP_FROM_G5)
                    if hasattr(dj_user, 'contact') and dj_user.contact:
                        dj_user.contact.groups.add(group)
                except Group.DoesNotExist:
                    pass

            if dj_user and password_is_valid:
                ldap_user.set_otp()
                ldap_user.save()
                return dj_user

        return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None


class ModelBackend(backends.ModelBackend):
    """
    Authenticates against django.contrib.auth.models.User.
    Differs from original in the fact that an empty password is rejected
    Django version raises an errore instead.
    """
    supports_inactive_user = True

    # TODO: Model, login attribute name and password attribute name should be
    # configurable.
    def authenticate(self, username=None, password=None):
        try:
            user = User.objects.get(username=username)
            if not user.password: # it's empty if the user was created from ldap
                return None
            if user.check_password(password):
                return user
        except User.DoesNotExist:
            return None


class AllAuthBackend(AuthenticationBackend):
    def authenticate(self, **credentials):
        return super(AllAuthBackend, self).authenticate(**credentials)

    def _authenticate_by_username(self, **credentials):
        return None

    def _authenticate_by_email(self, **credentials):
        email = credentials.get('email', credentials.get('username'))
        if email:
            # ci sono utenti senza password a seguito delle registrazioni LDAP
            for user in filter_users_by_email(email):
                if not user.password:
                    return None
                if user.check_password(credentials["password"]):
                    return user
        return None
